"""
filters for :mod:`django_notifications.admin` application.

:creationdate: 24/01/2022 14:19
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.admin.filters
"""
import logging
from typing import Any, Sequence, Tuple

from django.contrib.admin import SimpleListFilter
from django.db.models import QuerySet
from django.http import HttpRequest
from django.utils.translation import gettext_lazy as _

__author__ = "fguerin"

from django_notifications import models

logger = logging.getLogger(__name__)


class SentMessageListFilter(SimpleListFilter):
    """Filter messages by sent status."""

    title = _("Sent status")
    parameter_name = "sent"

    def lookups(
        self, request: HttpRequest, model_admin: Any
    ) -> Sequence[Tuple[str, str]]:
        """
        Return a list of tuples.

        :return: list of tuples
        """
        return (
            ("sent", _("Sent notifications")),
            ("unsent", _("Unsent notifications")),
        )

    def queryset(
        self, request: HttpRequest, queryset: QuerySet[models.Notification]
    ) -> QuerySet[models.Notification]:
        """
        Return the filtered queryset.

        :param request: HTTP Request
        :param queryset: queryset
        :return: Filtered queryset
        """
        if self.value() == "sent":
            return queryset.filter(sent_at__isnull=False)
        if self.value() == "unsent":
            return queryset.filter(sent_at__isnull=True)
        return queryset  # no filter


class DelayedMessageListFilter(SimpleListFilter):
    """Filter messages by delayed status."""

    title = _("Delayed status")
    parameter_name = "delayed"

    def lookups(
        self, request: HttpRequest, model_admin: Any
    ) -> Sequence[Tuple[str, str]]:
        """
        Return a list of tuples.

        :param request: HTTP request
        :param model_admin: model admin
        :return: list of tuples
        """
        return (
            ("delayed", _("Delayed notifications")),
            ("undelayed", _("Immediate notifications")),
        )

    def queryset(
        self, request: HttpRequest, queryset: QuerySet[models.Notification]
    ) -> QuerySet[models.Notification]:
        """
        Return the filtered queryset.

        :param request: HTTP request
        :param queryset: queryset
        :return: Filtered queryset
        """
        if self.value() == "delayed":
            return queryset.filter(delayed_sending_at__isnull=False)
        if self.value() == "undelayed":
            return queryset.filter(delayed_sending_at__isnull=True)
        return queryset  # no filter



# Generated by Django 3.2.9 on 2022-02-07 13:35

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_notifications", "0003_auto_20220207_1157"),
    ]

    operations = [
        migrations.AddField(
            model_name="notification",
            name="from_email",
            field=models.EmailField(
                default="evenements@ville-tourcoing.fr",
                help_text="Email address to use as sender",
                max_length=254,
                verbose_name="From email",
            ),
        ),
    ]



"""
send_notifications for :mod:`django_notifications.management.commands` application.

:creationdate:  07/01/2022 08:33
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.management.commands.send_notifications
"""
import logging
from typing import Any, Optional

from django.core.management.base import BaseCommand
from django.utils import timezone

from django_notifications import models

__author__ = "fguerin"
logger = logging.getLogger(__name__)


class Command(BaseCommand):
    """Send notifications command."""

    def handle(self, *args: Any, **options: Any) -> Optional[str]:
        """Send email notifications."""
        sent = 0
        for notification in models.Notification.objects.filter(
            sent_at__isnull=True
        ).filter(delayed_sending_at__lte=timezone.now()):
            sent += notification.send()
        self.stderr.write(f"{sent} notifications sent.\n")



# Generated by Django 3.2.12 on 2022-03-31 09:12

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_notifications", "0004_notification_from_email"),
    ]

    operations = [
        migrations.AddField(
            model_name="notificationconfig",
            name="debug_notifications_email",
            field=models.EmailField(
                blank=True,
                help_text="Email to send the notification copies to.",
                max_length=254,
                default="",
                verbose_name="Debug notifications email",
            ),
        ),
        migrations.AddField(
            model_name="notificationconfig",
            name="enable_debug_notifications",
            field=models.BooleanField(
                default=False,
                help_text="If enabled, notification copies will be sent to the provided email.",
                verbose_name="Enable debug notifications",
            ),
        ),
    ]



"""
Models for :mod:`django_notifications application`.

:creationdate: 06/01/2022 11:31
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.models
"""
import logging
import mimetypes
import pprint
from datetime import timedelta
from typing import List

from django.template.loader import render_to_string

from django_notifications.models import config, helpers

try:
    from functools import cached_property
except ImportError:  # python < 3.8
    from django.utils.functional import cached_property

from django.apps import apps
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.mail import EmailMultiAlternatives
from django.db import models
from django.db.models.fields.files import FieldFile
from django.utils import timezone
from django.utils.translation import gettext as _
from django_currentuser.middleware import get_current_authenticated_user

__author__ = "François GUÉRIN"
logger = logging.getLogger(__name__)


def filter_receive_notification(user: User) -> bool:
    """
    Filter users allowing to receive notification.

    :param user: User to filterS
    """
    _config = config.NotificationConfig.get_solo()
    _receive_notifications_field = _config.receive_notifications_field
    split = _receive_notifications_field.split(".")
    try:
        if len(split) == 1:
            _receive_notifications = getattr(user, _receive_notifications_field)
        else:
            _receive_notifications_profile = getattr(user, split[0])
            _receive_notifications = getattr(_receive_notifications_profile, split[1])
        logger.debug("filter_receive_notification(%s) %s", user, _receive_notifications)
        return _receive_notifications
    except AttributeError:
        logger.debug("filter_receive_notification(%s) No profile found !", user)
        return True


def filter_receive_messages(user) -> bool:
    """
    Filter users allowing to receive messages.

    :param user: User to filterS
    """
    _config = config.NotificationConfig.get_solo()
    _receive_messages_field = _config.receive_messages_field
    split = _receive_messages_field.split(".")
    try:
        if len(split) == 1:
            _receive_messages = getattr(user, _receive_messages_field)
        else:
            _receive_messages_profile = getattr(user, split[0])
            _receive_messages = getattr(_receive_messages_profile, split[1])
        logger.debug("filter_receive_messages(%s) %s", user, _receive_messages)
        return _receive_messages
    except AttributeError:
        logger.debug("filter_receive_message(%s) No profile found !", user)
        return True


class Attachment(models.Model):
    """Attachment for a notification."""

    #: The notification to attach the file to
    notification = models.ForeignKey(
        "django_notifications.Notification",
        on_delete=models.CASCADE,
        related_name="attachments",
    )

    #: The file to attach
    attachment_file = models.FileField(
        upload_to="notifications/attachments/%Y/%m/%d/",
    )

    class Meta:
        """Meta class."""

        verbose_name = _("Attachment")
        verbose_name_plural = _("Attachments")
        ordering = ("notification", "id")

    def __str__(self):
        """Get a string representation of the attachments."""
        return _("{attachment_file_name} for {notification}").format(
            attachment_file_name=self.attachment_file.name,
            notification=self.notification,
        )


class Notification(models.Model):
    """Model for :class:`django_notifications.models.Notification`."""

    # region Fields
    #: The user who created the notification
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_DEFAULT,
        default=1,
        editable=False,
        verbose_name=_("Created by"),
    )

    #: notification creation date
    created_at = models.DateTimeField(
        auto_now_add=True,
        editable=False,
        verbose_name=_("Created at"),
    )

    #: Optional delayed notification date
    delayed_sending_at = models.DateTimeField(
        verbose_name=_("Delayed sending at"),
        null=True,
        blank=True,
    )

    #: Effective notification sending date
    sent_at = models.DateTimeField(
        null=True,
        blank=True,
        editable=False,
        verbose_name=_("Sent at"),
    )

    #: Notification subject
    subject = models.CharField(
        max_length=255,
        verbose_name=_("Subject"),
    )

    #: Notification text content
    text_body = models.TextField(
        verbose_name=_("Text body"),
    )

    #: Notification html content
    html_body = models.TextField(
        verbose_name=_("HTML body"),
    )

    #: Notification related object content type
    content_type = models.ForeignKey(
        ContentType,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )

    #: Notification related object identifier
    object_id = models.PositiveIntegerField(
        null=True,
        blank=True,
    )

    #: Notification related object
    related_object = GenericForeignKey(
        "content_type",
        "object_id",
    )

    #: Notification object transition from state
    state_from = models.CharField(
        max_length=255,
        blank=True,
        default="",
        verbose_name=_("State from"),
    )

    #: Notification object transition to state
    state_to = models.CharField(
        max_length=255,
        blank=True,
        default="",
        verbose_name=_("State to"),
    )

    #: Recipients as users list
    recipients = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        verbose_name=_("Recipients"),
        related_name="notifications",
        blank=True,
    )

    #: Recipients as emails list, one per line
    email_recipients = models.TextField(
        verbose_name=_("Email recipients"),
        blank=True,
        help_text=_("One email per line"),
    )

    #: Email sender
    from_email = models.EmailField(
        verbose_name=_("From email"),
        help_text=_("Email address to use as sender"),
        default=settings.DEFAULT_FROM_EMAIL,
    )

    # endregion Fields

    class Meta:
        """Meta class for :class:`django_notifications.models.Notification`."""

        verbose_name = "Notification"
        verbose_name_plural = "Notifications"

    def __str__(self):
        """Get a string representation of the notification."""
        return f"{self.subject} - {self.created_by} - {self.created_at}"

    def _set_administrative_data(self):
        """
        Update the administrative fields of the Administrable models.

        .. note:: Hook method.

        :return: Nothing
        """
        current_user = get_current_authenticated_user()
        default_user = apps.get_model(settings.AUTH_USER_MODEL).objects.order_by("pk").first()  # type: ignore
        assert default_user is not None, "No default user found - please create one"

        if current_user is None:
            current_user = default_user
            logger.warning(
                "%s::_set_administrative_data() Unable to get the current user from local thread: "
                'setting the default one: "%s"',
                self.__class__.__name__,
                current_user,
            )

        # Looks like as nn instance creation
        if self.pk is None:
            self.created_by = current_user or default_user
            logging.info(
                "%s::_set_administrative_data() Creates a new instance (%s): created_by = %s",
                self.__class__.__name__,
                self,
                self.created_by,
            )

    def save_base(
        self,
        raw=False,
        force_insert=False,
        force_update=False,
        using=None,
        update_fields=None,
    ) -> None:
        """
        Save the connected user into the right field, according to the object state.

        + If the object is a new one: :attr:`django_notifications.models.Notification.created_by`
          is set to the current user.

        :param raw: Raw SQL query ?
        :param force_insert: Force insertion
        :param force_update: Force update
        :param using:  DB alias used
        :param update_fields: List fields to update
        :return: Nothing
        """
        self._set_administrative_data()

        super().save_base(
            raw=raw,
            force_insert=force_insert,
            force_update=force_update,
            using=using,
            update_fields=update_fields,
        )

    def get_recipients(self) -> List[str]:
        """Get **ALL** recipients for the email."""
        _recipients = []
        if self.email_recipients:
            _email_recipients = self.email_recipients
            if isinstance(_email_recipients, list):
                _recipients.extend(_email_recipients)
            elif isinstance(_email_recipients, str):
                _additional_recipients = [
                    r.strip() for r in _email_recipients.split("\n") if r.strip()
                ]
                _recipients.extend(_additional_recipients)
            else:
                logger.warning(
                    "%s::get_recipients() self.email_recipients not recognized: %s (%s)",
                    self.__class__.__name__,
                    self.email_recipients,
                    _email_recipients.__class__.__name__,
                )

        # Add the recipients from users, filtered by authorization
        if self.recipients.count() > 0:  # noqa
            filtered_users = filter(
                filter_receive_notification, self.recipients.all()
            )  # noqa
            _users = list(filtered_users)
            logger.debug(
                "%s::get_recipients() Filtered recipients: %s",
                self.__class__.__name__,
                pprint.pformat(_users),
            )
            _recipients.extend([user.email for user in _users])

        logger.info(
            "%s::get_recipients() Final recipient emails: %s",
            self.__class__.__name__,
            pprint.pformat(_recipients),
        )
        return _recipients

    @cached_property
    def email(self) -> EmailMultiAlternatives:
        """
        Get the email message.

        :return: Email message
        """
        recipients = self.get_recipients()

        # Create the email message
        email = EmailMultiAlternatives(
            subject=self.subject,
            body=self.text_body,
            from_email=self.from_email or settings.DEFAULT_FROM_EMAIL,
            to=recipients,
        )

        # Add HTML content
        if self.html_body:
            email.attach_alternative(self.html_body, "text/html")

        # Add the attachments (if needed)
        attachments = self.attachments.all()  # noqa
        if attachments.exists():
            for attachment in attachments:
                mime_type, _ = mimetypes.guess_type(attachment.attachment_file.name)
                email.attach_file(
                    path=attachment.attachment_file.path, mimetype=mime_type
                )
        return email

    @property
    def is_sent(self) -> bool:
        """
        Check if the notification is sent.

        :return: True if the notification is sent, False otherwise
        """
        return self.sent_at is not None

    def _add_attachment(self, attachment_file: FieldFile) -> Attachment:
        """Add a file to the notification."""
        attachment = Attachment.objects.create(  # noqa
            notification=self,
            attachment_file=attachment_file,
        )
        return attachment

    def notify(self, force: bool = False) -> int:
        """
        Notify the user by email.

        :param force: Force the notification
        :return: Number of sent messages
        """
        sent = self.notify_by_email(force=force)
        return sent

    def notify_by_email(self, force: bool = False) -> int:
        """
        Notify the user by email.

        :param force: Force the notification (A notification may be sent many times)
        :return: Number of sent messages
        """
        if self.is_sent and not force:
            logger.info(
                "%s::notify() {self} already sent at {self.sent_at} - skipping",
                self.__class__.__name__,
                self,
                self.sent_at,
            )
            return 0

        _config = config.NotificationConfig.get_solo()
        if _config.delay_notifications:
            logger.info(
                "%s::notify() %s delayed by %s seconds",
                self.__class__.__name__,
                self,
                _config.delay,
            )
            self.delayed_sending_at = timezone.now() + timedelta(seconds=_config.delay)
            return 0
        else:
            # Send the email
            sent = self.send()
            return sent

    def send(self, force: bool = False) -> int:
        """
        Send the email.

        ..note:: Actually send the email notification.

        :param force: Force the notification
        :return: Number of sent messages
        """
        if self.sent_at is not None and not force:
            logger.info(
                "%s::send() %s already sent at %s - skipping",
                self.__class__.__name__,
                self,
                self.sent_at,
            )
            return 0

        sent = self.email.send(fail_silently=False)
        # Update the sent_at field
        self.sent_at = timezone.now()
        self.save()

        # Send e copy of the email if required
        _config = config.NotificationConfig.get_solo()
        if _config.enable_debug_notifications:
            debug_mail = self.get_debug_email(to=_config.debug_notifications_email)
            debug_sent = debug_mail.send(fail_silently=False)
            logger.info(
                "%s::send() DEBUG %s sent %s emails to %s",
                self.__class__.__name__,
                self,
                debug_sent,
                _config.debug_notifications_email,
            )

        logger.info(
            "%s::send() %s sent %s emails for %s recipients",
            self.__class__.__name__,
            self,
            sent,
            self.recipients.count(),  # noqa
        )
        return sent

    def get_debug_email(self, to) -> EmailMultiAlternatives:
        """
        Get the email for debugging purposes.

        :param to: The email address to send the email to
        :return: The email
        """
        email = self.email
        _previous_to = email.to
        _previous_body = email.body
        _previous_html_body = email.alternatives[0][0]
        _new_to = [to]
        _new_body = render_to_string(
            "django_notifications/debug/body.txt.tpl",
            {
                "to": _previous_to,
                "content": _previous_body,
            },
        )
        _new_html_body = helpers.get_debug_html_body(
            html_body=_previous_html_body,
            to=_previous_to,
        )
        # Update the email with new data
        email.to = _new_to
        email.body = _new_body
        email.alternatives = [(_new_html_body, "text/html")]
        email.subject = f"DEBUG {self.subject}"
        return email



"""
Apps for :mod:`django_notifications` application.

:creationdate:  07/01/2022 10:33
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.apps
"""
from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DjangoNotificationsConfig(AppConfig):
    """AppConfig for :mod:`django_notifications` application."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "django_notifications"
    verbose_name = _("Notifications")



# Generated by Django 3.2.9 on 2022-02-07 10:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_notifications', '0002_notificationconfig'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notificationconfig',
            name='receive_messages_field',
            field=models.CharField(default='userprofile.receive_messages', help_text="Champs à vérifier sur le profil utilisateur pour valider si l'utilisateur souhaite recevoir les messages.", max_length=255, verbose_name='Nom du champs bloquant les messages'),
        ),
        migrations.AlterField(
            model_name='notificationconfig',
            name='receive_notifications_field',
            field=models.CharField(default='userprofile.receive_notifications', help_text="Champs à vérifier sur le profil utilisateur pour valider si l'utilisateur souhaite recevoir les notifications.", max_length=255, verbose_name='Nom du champs bloquant les notifications'),
        ),
    ]



"""
forms for :mod:`django_notifications.admin` application.

:creationdate:  07/01/2022 10:33
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.admin.forms
"""
import logging

from django import forms

from django_notifications import models

__author__ = "fguerin"
logger = logging.getLogger(__name__)


class NotificationAdminForm(forms.ModelForm):
    """Admin form for :class:`django_notifications.models.Notification` model."""

    class Meta:
        """Meta class for :class:`django_notifications.admin.NotificationAdminForm`."""

        model = models.Notification
        fields = "__all__"



"""
inlines for :mod:`django_notifications.admin` application.

:creationdate:  07/01/2022 10:36
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.admin.inlines
"""
import logging

from django.contrib import admin

from django_notifications import models

__author__ = "fguerin"
logger = logging.getLogger(__name__)


class AttachmentInline(admin.TabularInline):
    """Inline for :class:`django_notifications.models.Attachment`."""

    model = models.Attachment
    extra = 1



"""
config for :mod:`django_notifications.models` application.

:creationdate: 31/03/2022 11:53
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.models.config
"""
import logging

__author__ = "fguerin"

from django.db import models
from django.utils.translation import gettext as _
from solo.models import SingletonModel

logger = logging.getLogger(__name__)


class NotificationConfig(SingletonModel):
    """Configuration for the notifications."""

    #: If set, the notification will be delayed and sent through a cron job
    delay_notifications = models.BooleanField(
        verbose_name=_("Delay notifications"),
        default=False,
        help_text=_("Delay the notifications, using a cron job."),
    )

    #: Delay to apply to the notifications
    delay = models.PositiveIntegerField(
        verbose_name=_("Delay"),
        default=0,
        help_text=_("Delay in seconds."),
    )

    #: If set, the notification will be sent to the user.
    receive_notifications_field = models.CharField(
        verbose_name=_("Blocking notifications field"),
        max_length=255,
        default="userprofile.receive_notifications",
        help_text=_("Field to check if the user wants to receive notifications."),
    )

    #: If set, the messages will be sent to the user.
    receive_messages_field = models.CharField(
        verbose_name=_("Blocking messages field"),
        max_length=255,
        default="userprofile.receive_messages",
        help_text=_("Field to check if the user wants to receive messages."),
    )

    enable_debug_notifications = models.BooleanField(
        verbose_name=_("Enable debug notifications"),
        default=False,
        help_text=_(
            "If enabled, notification copies will be sent to the provided email."
        ),
    )

    debug_notifications_email = models.EmailField(
        verbose_name=_("Debug notifications email"),
        blank=True,
        default="",
        help_text=_("Email to send the notification copies to."),
    )

    class Meta:
        """Meta class."""

        verbose_name = _("Notification configuration")
        verbose_name_plural = _("Notification configurations")

    def __str__(self):
        """Get the string representation."""
        return _("Notification configuration")



"""
helpers for :mod:`django_notifications.models` application.

:creationdate: 31/03/2022 14:08
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.models.helpers
"""
import logging
from typing import List

import bs4
from bs4.element import Tag

__author__ = "fguerin"
logger = logging.getLogger(__name__)


def get_debug_head(soup: bs4.BeautifulSoup, to: List[str]) -> Tag:
    """
    Get the DEBUG head for email debugging.

    :param soup: html body to convert
    :param to: list of recipients
    :return: html head
    """

    def get_email_line(i: int, email: str) -> Tag:
        """Create a line for the email."""
        new_tag = soup.new_tag("li", id=f"id_to_list_item_{i:04d}")
        new_tag.string = email
        return new_tag

    debug_tag = soup.new_tag(
        "div",
        id="id_debug",
        style="display: block; border: 1px solid red; background-color: #ffcccc;",
    )
    to_list_tag = soup.new_tag("ul", id="id_to_list")
    to_list_tag.contents.extend(
        [get_email_line(i, email) for i, email in enumerate(to)]
    )
    title_tag = soup.new_tag("h3")
    title_tag.string = "DEBUG"
    debug_tag.contents = [
        title_tag,
        to_list_tag,
    ]
    return debug_tag


def get_debug_html_body(html_body: str, to: List[str]) -> str:
    """
    Convert html body for email debugging.

    :param html_body: html body to convert
    :param to: list of recipients
    :return: html body with <pre> tags
    """
    soup = bs4.BeautifulSoup(html_body, "html.parser")
    if soup is None or soup.body is None:
        logger.warning("get_debug_html_body() Unable to parse html body")
        return html_body
    debug_tag = get_debug_head(soup, to)
    soup.body.insert(0, debug_tag)
    return soup.decode(pretty_print=True)



"""
Tests for :mod:`django_notifications` project.

:creationdate: 01/01/2022 10:46
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.tests
"""
from django.apps import apps
from django.conf import settings
from django.test import TestCase

from django_notifications import models


class TestDjangoNotifications(TestCase):
    """Test the notification models."""

    def test_creation_with_user(self):
        """Test the creation of a notification."""
        user = apps.get_model(settings.AUTH_USER_MODEL)
        notification = models.objects.create(
            subject="Test",
            text_body="Test",
            html_body="<p>Test</p>",
            recipients=[user],
        )

        self.assertIsNotNone(notification)
        self.assertIsNotNone(notification.created_by)
        self.assertEqual(notification.created_by, user)
        self.assertIs(notification.recipients.all().exists(), True)



"""
Models for :mod:`django_notifications application`.

:creationdate: 06/01/2022 11:31
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.models
"""
import logging
from typing import Optional

from django.conf import settings
from django.contrib import admin, messages
from django.db.models import QuerySet
from django.http import HttpRequest
from django.template.loader import render_to_string
from django.urls import reverse_lazy
from django.utils.safestring import SafeString, mark_safe
from django.utils.translation import gettext_lazy as _
from solo import admin as solo_admin

import django_notifications.models.config
from django_notifications import models

from . import forms, inlines
from .filters import DelayedMessageListFilter, SentMessageListFilter

logger = logging.getLogger(__name__)
__author__ = "François GUÉRIN"


@admin.register(models.Notification)
class NotificationAdmin(admin.ModelAdmin):
    """Admin for :class:`django_notifications.models.Notification`."""

    list_display = (
        "get_subject",
        "get_dates",
        "get_related_object",
        "get_states",
        "get_recipients",
    )

    date_hierarchy = "created_at"

    list_filter = [
        SentMessageListFilter,
        DelayedMessageListFilter,
    ]

    inlines = [
        inlines.AttachmentInline,
    ]

    form = forms.NotificationAdminForm

    fieldsets = (
        (
            _("Notification"),
            {
                "fields": (
                    "subject",
                    "text_body",
                    "html_body",
                    "state_from",
                    "state_to",
                )
            },
        ),
        (
            _("Related object"),
            {"fields": ("content_type", "object_id")},
        ),
        (
            _("Recipients"),
            {
                "fields": (
                    "recipients",
                    "email_recipients",
                )
            },
        ),
        (_("Delayed"), {"fields": ("delayed_sending_at",)}),
    )

    search_fields = [
        "email_recipients",
        "recipients__email",
    ]

    # region Permissions
    def has_add_permission(self, request: HttpRequest) -> bool:
        """
        Check if user has permission to add a notification.

        .. note:: Always return ``False``.
        """
        return False

    def has_delete_permission(
        self, request: HttpRequest, obj: Optional[models.Notification] = None
    ) -> bool:
        """
        Check if user has permission to delete a notification.

        .. note:: Return `True` for super users.
        """
        user = request.user
        return user.is_superuser

    # endregion Permissions

    # region List display methods
    @admin.display(description=_("Subject"))
    def get_subject(self, obj: models.Notification) -> SafeString:
        """
        Get the subject and the `sent` status.

        :param obj: The notification object
        :return: The subject and the `sent` status.
        """
        return mark_safe(
            render_to_string(
                "django_notifications/admin/snippets/subject.html",
                {
                    "debug": settings.DEBUG,
                    "object": obj,
                },
            )
        )

    @admin.display(description=_("Dates"), ordering="created_at")
    def get_dates(self, obj: models.Notification) -> SafeString:
        """
        Display dates.

        :param obj: :class:`django_notifications.models.Notification` instance
        :return: rendered HTML
        """
        return mark_safe(
            render_to_string(
                "django_notifications/admin/snippets/dates.html",
                {
                    "debug": settings.DEBUG,
                    "object": obj,
                },
            )
        )

    @admin.display(description=_("Related object"))
    def get_related_object(self, obj: models.Notification) -> Optional[SafeString]:
        """
        Display related object.

        :param obj: :class:`django_notifications.models.Notification` instance
        :return: rendered HTML
        """
        if obj.related_object:
            admin_url = reverse_lazy(
                "admin:{app_label}_{model_name}_change".format(
                    app_label=obj.related_object._meta.app_label,  # noqa
                    model_name=obj.related_object._meta.model_name,  # noqa
                ),
                args=(obj.related_object.pk,),
            )
            return mark_safe(
                render_to_string(
                    "django_notifications/admin/snippets/related_object.html",
                    {
                        "debug": settings.DEBUG,
                        "object": obj,
                        "admin_url": admin_url,
                    },
                )
            )
        return None

    @admin.display(description=_("States"), ordering="state_to")
    def get_states(self, obj) -> SafeString:
        """
        Display states.

        :param obj: :class:`django_notifications.models.Notification` instance
        :return: rendered HTML
        """
        return mark_safe(
            render_to_string(
                "django_notifications/admin/snippets/states.html",
                {
                    "debug": settings.DEBUG,
                    "object": obj,
                },
            )
        )

    @admin.display(description=_("Recipients"))
    def get_recipients(self, obj) -> SafeString:
        """
        Display recipients.

        :param obj: :class:`django_notifications.models.Notification` instance
        :return: rendered HTML
        """
        return mark_safe(
            render_to_string(
                "django_notifications/admin/snippets/recipients.html",
                {
                    "debug": settings.DEBUG,
                    "object": obj,
                },
            )
        )

    # endregion List display methods

    # region Action methods
    @admin.action(description=_("Send notifications"))
    def send_notifications(
        self, request: HttpRequest, queryset: QuerySet[models.Notification]
    ) -> None:
        """
        Re-send notifications.

        :param request HTTP request
        :param queryset: Selected notifications
        :return: Nothing
        """
        for notification in queryset:
            _sent = notification.notify(force=True)
            if _sent:
                messages.info(
                    request=request,
                    message=_(
                        "A notification has been sent for {notification}."
                    ).format(notification=notification),
                )
            else:
                messages.warning(
                    request=request,
                    message=_("No notification sent for {notification}.").format(
                        notification=notification
                    ),
                )

    actions = [
        send_notifications,
    ]
    # endregion Action methods


@admin.register(models.Attachment)
class AttachmentAdmin(admin.ModelAdmin):
    """Admin for :class:`django_notifications.models.Attachment`."""

    list_display = [
        "id",
        "notification",
        "attachment_file",
    ]

    def has_add_permission(self, request: HttpRequest) -> bool:  # noqa
        """
        Don't allow adding attachments.

        :param request: HTTP request
        :return: False
        """
        return False

    def has_delete_permission(
        self, request: HttpRequest, obj: models.Attachment = None
    ) -> bool:  # noqa
        """
        Don't allow deleting attachments.

        :param request: HTTP request
        :param obj: :class:`django_notifications.models.Attachment` instance
        :return: False
        """
        return False

    def has_change_permission(
        self, request: HttpRequest, obj: models.Attachment = None
    ) -> bool:  # noqa
        """
        Don't allow changing attachments.

        :param request: HTTP request
        :param obj: :class:`django_notifications.models.Attachment` instance
        :return: False
        """
        return False


@admin.register(django_notifications.models.config.NotificationConfig)
class NotificationConfigAdmin(solo_admin.SingletonModelAdmin):
    """Admin for :class:`django_notifications.models.NotificationConfig`."""

    fieldsets = (
        (
            None,
            {"fields": ("receive_notifications_field", "receive_messages_field")},
        ),
        (
            _("Delayed notifications"),
            {"fields": ("delay_notifications", "delay")},
        ),
        (
            _("Debug"),
            {
                "classes": ("collapse",),
                "fields": ("enable_debug_notifications", "debug_notifications_email"),
            },
        ),
    )



# Generated by Django 3.2.11 on 2022-01-14 07:31

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_notifications', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='NotificationConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('delay_notifications', models.BooleanField(default=False, help_text='Retarder les notifications en utilisant une tâche cron .', verbose_name='Retarder les notifications')),
                ('delay', models.PositiveIntegerField(default=0, help_text='Délais en secondes', verbose_name='Délais')),
                ('receive_notifications_field', models.CharField(default='userprofile.receive_notifications', help_text='Field to check if the user wants to receive notifications.', max_length=255, verbose_name='Blocking notifications field')),
                ('receive_messages_field', models.CharField(default='userprofile.receive_messages', help_text='Field to check if the user wants to receive messages.', max_length=255, verbose_name='Blocking messages field')),
            ],
            options={
                'verbose_name': 'Configuration des notifications',
                'verbose_name_plural': 'Configurations des notifications',
            },
        ),
        migrations.AlterModelOptions(
            name='attachment',
            options={'ordering': ('notification', 'id'), 'verbose_name': 'Pièce jointe', 'verbose_name_plural': 'Pièces jointes'},
        ),
        migrations.AlterField(
            model_name='notification',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Ajouté le'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='created_by',
            field=models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.SET_DEFAULT, to=settings.AUTH_USER_MODEL, verbose_name='Ajouté par'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='delayed_sending_at',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Envoi différé'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='email_recipients',
            field=models.TextField(blank=True, help_text='Une adresse par ligne', verbose_name='Adresses électroniques de destinataires'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='html_body',
            field=models.TextField(verbose_name='Corps du message, en HTML'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='recipients',
            field=models.ManyToManyField(blank=True, related_name='notifications', to=settings.AUTH_USER_MODEL, verbose_name='Destinataires'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='sent_at',
            field=models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Envoyé le'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='state_from',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='État de départ'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='state_to',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name="État d'arrivée"),
        ),
        migrations.AlterField(
            model_name='notification',
            name='subject',
            field=models.CharField(max_length=255, verbose_name='Sujet'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='text_body',
            field=models.TextField(verbose_name='Corps du message, en texte'),
        ),
    ]



"""
notifications_tags for :mod:`django_notifications.templatetags` application.

:creationdate: 31/03/2022 15:08
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications.templatetags.notifications_tags
"""
import logging
import warnings

from django import template
from django.utils.safestring import SafeString, mark_safe

register = template.Library()
__author__ = "fguerin"
logger = logging.getLogger(__name__)


@register.filter(name="split")
def split(value, arg):
    """
    Split a string by a separator.

    :param value: The string to split
    :param arg: The separator
    :return: The splited string
    """
    return value.split(arg)


@register.simple_tag(name="spaces", takes_context=False)
def get_spaces(count: int = 4) -> SafeString:
    """
    Return a string of spaces.

    :param count: Number of spaces
    :return: String of spaces
    """
    warnings.warn(
        "`spaces` is deprecated, use `ljust` filter instead", DeprecationWarning
    )
    return mark_safe(" " * count)



"""
Application to manage notifications.

:copyright: (c) 2022 by François GUÉRIN
:license: mit, see LICENSE for more details.
:creationdate: 06/01/2022 11:31
:moduleauthor: François GUÉRIN <fguerin@ville-tourcoing.fr>
:modulename: django_notifications

"""
__version__ = VERSION = "0.6.17"



# Generated by Django 3.2.11 on 2022-01-07 07:52

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Notification",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="Created at")),
                ("delayed_sending_at", models.DateTimeField(blank=True, null=True, verbose_name="Delayed sending at")),
                ("sent_at", models.DateTimeField(blank=True, editable=False, null=True, verbose_name="Sent at")),
                ("subject", models.CharField(max_length=255, verbose_name="Subject")),
                ("text_body", models.TextField(verbose_name="Text body")),
                ("html_body", models.TextField(verbose_name="HTML body")),
                ("object_id", models.PositiveIntegerField(blank=True, null=True)),
                ("state_from", models.CharField(blank=True, default="", max_length=255, verbose_name="State from")),
                ("state_to", models.CharField(blank=True, default="", max_length=255, verbose_name="State to")),
                (
                    "email_recipients",
                    models.TextField(blank=True, help_text="One email per line", verbose_name="Email recipients"),
                ),
                (
                    "content_type",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="contenttypes.contenttype",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=1,
                        editable=False,
                        on_delete=django.db.models.deletion.SET_DEFAULT,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Created by",
                    ),
                ),
                (
                    "recipients",
                    models.ManyToManyField(
                        blank=True, related_name="notifications", to=settings.AUTH_USER_MODEL, verbose_name="Recipients"
                    ),
                ),
            ],
            options={
                "verbose_name": "Notification",
                "verbose_name_plural": "Notifications",
            },
        ),
        migrations.CreateModel(
            name="Attachment",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("attachment_file", models.FileField(upload_to="notifications/attachments/%Y/%m/%d/")),
                (
                    "notification",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="attachments",
                        to="django_notifications.notification",
                    ),
                ),
            ],
            options={
                "verbose_name": "Attachment",
                "verbose_name_plural": "Attachments",
                "ordering": ("notification", "id"),
            },
        ),
    ]
